"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("../");
describe("Try", () => {
    describe("Try.success", () => {
        test("Try.success should create a Success instance", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success("test");
            expect(yield result.get()).toBe("test");
            expect(result.isSuccess()).toBe(true);
        }));
    });
    describe("Try.failure", () => {
        test("Try.failure should create a Failure instance", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error"));
            yield expect(result.get()).rejects.toThrow("test error");
            expect(result.isFailure()).toBe(true);
        }));
    });
    describe("Try.get", () => {
        test("get should return the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2);
            expect(yield result.get()).toBe(2);
        }));
        test("get should throw the error inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error"));
            yield expect(result.get()).rejects.toThrow("test error");
        }));
    });
    describe("Try.getOrElse", () => {
        test("getOrElse should return the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2);
            expect(yield result.getOrElse(4)).toBe(2);
        }));
        test("getOrElse should return the default value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error"));
            expect(yield result.getOrElse(4)).toBe(4);
        }));
    });
    describe("Try.getOrElseGet", () => {
        test("getOrElseGet should return the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2);
            expect(yield result.getOrElseGet(() => 4)).toBe(2);
        }));
        test("getOrElseGet should return the default value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error"));
            expect(yield result.getOrElseGet(() => 4)).toBe(4);
        }));
    });
    describe("Try.getOrElseThrow", () => {
        test("getOrElseThrow should return the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2);
            expect(yield result.getOrElseThrow(() => new Error("test error"))).toBe(2);
        }));
        test("getOrElseThrow should throw the custom error inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error"));
            yield expect(result.getOrElseThrow(() => new Error("custom error"))).rejects.toThrow("custom error");
        }));
    });
    describe("Try.of", () => {
        test("Try.of should create a Success instance when no exception is thrown", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.of(() => "test");
            yield expect(result.get()).resolves.toBe("test");
            expect(result.isSuccess()).toBe(true);
        }));
        test("Try.of should create a Failure instance when an exception is thrown", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.of(() => { throw new Error("test error"); });
            yield expect(result.get()).rejects.toThrow("test error");
            expect(result.isFailure()).toBe(true);
        }));
    });
    describe("Try.map", () => {
        test("map should transform the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2)
                .map(v => v * 2);
            yield expect(result.get()).resolves.toBe(4);
            expect(result.isSuccess()).toBe(true);
        }));
        test("map should not transform the value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            const result = __1.Try.failure(new Error("test error")).map(v => v * 2);
            yield expect(result.get()).rejects.toThrow("test error");
            expect(result.isFailure()).toBe(true);
        }));
    });
    describe("Try.flatMap", () => {
        test("flatMap should transform the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).flatMap(v => __1.Try.success(v * 2));
            yield expect(result.get()).resolves.toBe(4);
            expect(result.isSuccess()).toBe(true);
        }));
        test("flatMap should not transform the value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            // @ts-ignore
            const result = __1.Try.failure(new Error("test error")).flatMap(v => __1.Try.success(v * 2));
            yield expect(() => result.get()).rejects.toThrow("test error");
            expect(result.isFailure()).toBe(true);
        }));
    });
    describe("Try.filter", () => {
        test("filter should return Failure if predicate does not hold", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filter(v => v > 2);
            yield expect(result.get()).rejects.toThrow("Predicate does not hold for 2");
            expect(result.isFailure()).toBe(true);
        }));
        test("filter should throw custom exception if predicate does not hold", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filter(v => v > 2, v => { throw new Error("Custom Predicate does not hold for " + v); });
            yield expect(result.get()).rejects.toThrow("Custom Predicate does not hold for 2");
            expect(result.isFailure()).toBe(true);
        }));
        test("filter should return Success if predicate holds", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filter(v => v <= 2);
            yield expect(result.get()).resolves.toBe(2);
            expect(result.isSuccess()).toBe(true);
        }));
    });
    describe("Try.filterNot", () => {
        test("filterNot should return Failure if predicate does not hold", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filterNot(v => v <= 2);
            yield expect(result.get()).rejects.toThrow("Predicate does not hold for 2");
            expect(result.isFailure()).toBe(true);
        }));
        test("filterNot should throw custom exception if predicate does not hold", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filterNot(v => v <= 2, v => { throw new Error("Custom Predicate does not hold for " + v); });
            yield expect(result.get()).rejects.toThrow("Custom Predicate does not hold for 2");
            expect(result.isFailure()).toBe(true);
        }));
        test("filterNot should return Success if predicate holds", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).filterNot(v => v > 2);
            yield expect(result.get()).resolves.toBe(2);
            expect(result.isSuccess()).toBe(true);
        }));
    });
    describe("Try.peek", () => {
        test("should print out the current value in the chain", () => __awaiter(void 0, void 0, void 0, function* () {
            let tempResult = 0;
            const result = __1.Try.success(2).map(value => value * 2).peek(v => { tempResult = v; }).map(v => v * 2);
            yield expect(result.get()).resolves.toBe(8);
            expect(tempResult).toBe(4);
            expect(result.isSuccess()).toBe(true);
        }));
        test("should convert to failure if peek function throws", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).map(value => value * 2).peek(_ => { throw new Error("Thrown in peek function"); }).map(v => v * 2);
            yield expect(result.get()).rejects.toThrow("Thrown in peek function");
            expect(result.isFailure()).toBe(true);
        }));
    });
    describe("Try.recover", () => {
        test("recover should transform the value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error")).recover(_ => "Recovered");
            yield expect(result.get()).resolves.toBe("Recovered");
            expect(result.isSuccess()).toBe(true);
        }));
        test("recover should not transform the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).recover(_ => "Recovered");
            yield expect(result.get()).resolves.toBe(2);
            expect(result.isSuccess()).toBe(true);
        }));
    });
    describe("Try.recoverWith", () => {
        test("recoverWith should transform the value inside Failure", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.failure(new Error("test error")).recoverWith(_ => __1.Try.failure(new Error("Failure")).recover(_ => "Recovered from inside"));
            yield expect(result.get()).resolves.toBe("Recovered from inside");
            expect(result.isSuccess()).toBe(true);
        }));
        test("recoverWith should not transform the value inside Success", () => __awaiter(void 0, void 0, void 0, function* () {
            const result = __1.Try.success(2).recoverWith(_ => __1.Try.success("Recovered"));
            yield expect(result.get()).resolves.toBe(2);
            expect(result.isSuccess()).toBe(true);
        }));
    });
});
